#!/usr/bin/python3
# DistUpgradeConfigParser.py 
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#  USA


from configparser import NoOptionError, NoSectionError
from configparser import ConfigParser as SafeConfigParser
import subprocess
import os.path
import logging
import glob

CONFIG_OVERRIDE_DIR = "/var/lib/kylin-system-updater/upgrades.d"


class UpgradeConfig(SafeConfigParser):
    def __init__(self, datadir, name="system-updater.conf", 
                 override_dir=None, defaults_dir=None):
        SafeConfigParser.__init__(self)
        # we support a config overwrite, if DistUpgrade.cfg.dapper exists
        # and the user runs dapper, that one will be used
        self.datadir = datadir
        maincfg = os.path.join(datadir, name)
        # defaults are read first
        self.config_files = []
        if defaults_dir:
            for cfg in glob.glob(defaults_dir + "/*.cfg"):
                self.config_files.append(cfg)
        # our config file
        self.config_files += [maincfg]
        # overrides are read later
        if override_dir is None:
            override_dir = CONFIG_OVERRIDE_DIR
        if override_dir is not None:
            for cfg in glob.glob(override_dir + "/*.cfg"):
                self.config_files.append(cfg)
        self.read(self.config_files)
        logging.info("Initialize Upgrade ConfigFile(%s) to success",str(self.config_files))

    def reReadConfigFiles(self):
        self.read(self.config_files)

    def setValue(self, section, option, value=None,is_write = False):
        if option != 'upgradelist':
            logging.info("setValue section:%s option:%s value:%s",section, option, value)
        self.set(section, option,value)
        if is_write == True:
            with open(self.config_files[0], 'w+') as configfile:
                self.write(configfile)

    def getWithDefault(self, section, option, default):
        try:
            if type(default) == bool:
                return self.getboolean(section, option)
            elif type(default) == float:
                return self.getfloat(section, option)
            elif type(default) == int:
                return self.getint(section, option)
            return self.get(section, option)
        except (NoSectionError, NoOptionError):
            return default

    def getlist(self, section, option):
        try:
            tmp = self.get(section, option)
        except (NoSectionError, NoOptionError):
            return []
        items = [x.strip() for x in tmp.split(",")]
        return items

    def getListFromFile(self, section, option):
        try:
            filename = self.get(section, option)
        except NoOptionError:
            return []
        p = os.path.join(self.datadir, filename)
        if not os.path.exists(p):
            logging.error("getListFromFile: no '%s' found" % p)
        with open(p) as f:
            items = [x.strip() for x in f]
        return [s for s in items if not s.startswith("#") and not s == ""]


if __name__ == "__main__":
    # c = UpgradeConfig("/home/x/share/kylin-system-updater/backend/data/")
    # print(c.setValue("SystemStatus", "isabnormalreboot", str(False)),True)
    # print(c.getWithDefault("SystemStatus", "isabnormalreboot", False))
    pass
