#!/usr/bin/python3

import os
import apt
import logging
import apt_pkg
from defer import inline_callbacks
from apt.progress.base import AcquireProgress
from gettext import gettext as _

#python-apt download only
class FetchProgress(AcquireProgress):
    def __init__(self, downdeamon) -> None:
        super().__init__()
        self.downd = downdeamon
        self.dbusDeamon = downdeamon.window_main.dbusController
        self.kargs = {"desc":"", "err_text":"","status":"", "type":""}
        self.time = 0
        

    def pulse(self, owner):
        progress = int((self.current_bytes / self.total_bytes) * 100)
        total_size = self.total_bytes
        pkgname = ''
        tmplist = []
        current_cps =0
        if owner.workers[0].current_item != None:
            pkgname=str(owner.workers[0].current_item.shortdesc)
            for i in owner.items:
                allpkgs=os.path.basename(i.destfile)
                allpkgs = allpkgs.split("_")[0]
                tmplist.append(allpkgs)
            current_items = tmplist.index(pkgname)
        else:
            current_items = 0
        current_cps = self.calculatcps()
        if current_cps == None:
            current_cps = 0
        self.dbusDeamon.UpdateDownloadInfo(\
                            current_items,  len(owner.items), \
                            self.current_bytes, owner.total_needed, \
                            current_cps)
        return

    def stop(self):
        logging.info("fetch progess finished")
        finish_status = False
        if self.current_items == self.total_items:
            finish_status = True
        else:
            finish_status = False
        error_string = "Download " + self.kargs['desc'] + self.kargs['type']
        error_desc   = self.kargs['err_text']
        self.dbusDeamon.UpdateInstallFinished(finish_status, self.downd.pkglists, error_string, error_desc)
        return

    def fail(self, acquireitemdesc):
        desc = acquireitemdesc.shortdesc
        err_text = acquireitemdesc.owner.error_text
        status = acquireitemdesc.owner.status
        type = ""
        if status == apt_pkg.AcquireItem.STAT_AUTH_ERROR:
            type = "authentication error"
        elif status == apt_pkg.AcquireItem.STAT_ERROR:
            type = "fetch error"
        elif status == apt_pkg.AcquireItem.STAT_TRANSIENT_NETWORK_ERROR:
            type = "network error"
        elif status == apt_pkg.AcquireItem.STAT_IDLE:
            type = "to be fetched"
        elif status == apt_pkg.AcquireItem.STAT_FETCHING:
            type = "fetching"
        elif status == apt_pkg.AcquireItem.STAT_DONE:
            type = "finished"
        else:
            type = "normal"
        self.kargs = {"desc":desc, \
                      "err_text":err_text, \
                      "status":status, \
                      "type":type}
        # self.deamon.DownloadErrorSignal(desc, type, err_text)
        return
    
    def start(self):
        # type: () -> None
        """Invoked when the Acquire process starts running."""
        # Reset all our values.
        self.current_bytes = 0.0
        self.current_cps = 0.0
        self.current_items = 0
        self.elapsed_time = 0
        self.fetched_bytes = 0.0
        self.last_bytes = 0.0
        self.total_bytes = 0.0
        self.total_items = 0
        logging.info("start download ...")
    
    def calculatcps(self):
        import datetime
        if self.time == 0:
            self.time = datetime.datetime.now()
            self.currentbyte = self.current_bytes
            return 0
        else:
            time = datetime.datetime.now()
            if self.current_bytes == self.currentbyte:
                cps = 0
            else:
                cps = (self.current_bytes-self.currentbyte) / (time-self.time).microseconds * 1000000
                # print((time-self.time).microseconds)
                # print(self.current_bytes-self.currentbyte)
                self.currentbyte = self.current_bytes
                self.time = time
                return cps

class DownloadBackend():
    def __init__(self, window_main):
        self.window_main = window_main
        self.pkglists = []

    def downloadpkgs(self, pkgs = []):
        logging.info("Pkgs: %s", pkgs)
        self.pkglists = pkgs
        apt_pkg.init()
        cache = apt_pkg.Cache()
        depcache = apt_pkg.DepCache(cache)
        packagerecords = apt_pkg.PackageRecords(cache)
        pm = apt_pkg.PackageManager(depcache)
        sourcelist = apt_pkg.SourceList()
        sourcelist.read_main_list()
        try:
            for pkgname in pkgs:
                pkg = cache[pkgname]
                depcache.mark_install(pkg)
        except Exception as e:
            logging.error(str(e))
            return False
        fetcher = apt_pkg.Acquire(FetchProgress(self))
        pm.get_archives(fetcher, sourcelist, packagerecords)
        ret = fetcher.run()

